/****************************************************************************
//  Created by Ben Schmidt on Jan 18, 2017
//  Copyright (c) 2017 Noodlecake Studios Inc. All rights reserved.
 ****************************************************************************/

package com.noodlecake.BinkANE;

import java.util.HashMap;
import java.util.Map;

import com.adobe.fre.FREContext;
import com.adobe.fre.FREFunction;
import com.adobe.air.ActivityResultCallback;  
import com.adobe.air.AndroidActivityWrapper;  
import com.adobe.air.AndroidActivityWrapper.ActivityState;  
import com.adobe.air.StateChangeCallback; 

import com.noodlecake.BinkANE.Extension;
import android.util.Log;

import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.content.res.Configuration; 
import android.content.Intent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager.LayoutParams;
import android.view.ActionMode;
import android.view.ActionMode.Callback;
import android.view.MotionEvent;
import android.view.KeyEvent;
import android.view.SearchEvent;
import android.view.accessibility.AccessibilityEvent;
import android.view.Menu;
import android.view.MenuItem;

import com.noodlecake.BinkANE.functions.*;

import com.noodlecake.NoodleX.NoodleXBridge;

//Native functions
public class ExtensionContext extends FREContext implements ActivityResultCallback, StateChangeCallback
{

    private AndroidActivityWrapper aaw;  
    private boolean isCreated = false;
    
    public ExtensionContext() {
        aaw = AndroidActivityWrapper.GetAndroidActivityWrapper();  
        aaw.addActivityResultListener(this);  
        aaw.addActivityStateChangeListner(this);
    }

	@Override
	public void dispose() { 
        if (aaw!=null) {  
            aaw.removeActivityResultListener(this);  
            aaw.removeActivityStateChangeListner(this);  
            aaw = null;  
        }  
    }

	@Override
	public Map<String, FREFunction> getFunctions()
	{
		Map<String, FREFunction> functionMap = new HashMap<String, FREFunction>();
		
        /*** INIT ***/

        //initializes the BinkController class, sets default values, gets some device information
        functionMap.put("icInitialise", new icInitialise()); 

        //checks if we're using an obb, if we have said obb, and if there is a cloud save available
        functionMap.put("checkStartupData", new checkStartupData());

        //returns true if the build is using an obb, false otherwise
        functionMap.put("usingObb", new usingObb());

        /*** ASSET LOADING ***/
        //These calls are needed to get the game's assets so that adobe air can use them.

        //Finds asset and returns the URL
        functionMap.put("getAssetUrl", new getAssetUrl());

        //Given a particular directory, finds asset and returns the URL
        functionMap.put("getAssetUrlInDir", new getAssetUrlInDir());

        //Loads basic image files - mostly jpg - RGB and RGBA
        functionMap.put("loadImageFor", new loadImageFor());

        //Gets info about the loaded image
        functionMap.put("getLoadedImageInfo", new getLoadedImageInfo());

        //Gets the loaded image bitmap data
        functionMap.put("getLoadedImage", new getLoadedImage());

        //Loads big graphics - mostly background images
        //These are the atf files in the game
        //L for loading, U for usage (SALGPUFill vs SAUGPUFill)
        functionMap.put("getBytesFor", new getBytesFor());


        /*** BINK VIDEO ***/
        //These calls are used to control the background videos, which are run by the Bink SDK.
        //The basic approach is that they get the GLView that Adobe Air is using and render the video frames
        //in the background of that view before Adobe renders its elements.
        
        //initializes the Bink SDK
        functionMap.put("initBink", new initBink());

        //reinitializes the Bink SDK after the app has been backgrounded
        functionMap.put("reinitBink", new reinitBink());

        //Sets the extension for videos in the game - based on screen size/ratio
        //Ex. On iOS, the .bik files can end in:
        //    4b3h (4:3 high quality)
        //    4b3l (4:3 low qualit7)
        //    16b9h (16:9 high quality)
        //    16b9l (16:9 low quality)
        functionMap.put("setVideoSuffix", new setVideoSuffix());
        
        //tells Bink to load a video
        functionMap.put("loadVideo", new loadVideo());

        //gets total number of frames in the current Bink video
        functionMap.put("getTotalFrames", new getTotalFrames());

        //gets the next frame of the current Bink video
        functionMap.put("nextFrame", new nextFrame());

        //goes to a particular frame of the current Bink video
        functionMap.put("gotoFrame", new gotoFrame());

        //sets the size of the quad to draw a Bink video frame on
        functionMap.put("setdraw", new setdraw());

        //draws the current frame of the Bink video
        functionMap.put("draw", new draw());

        //experimental - not used - shouldn't need to implement for Android
        functionMap.put("loadDualVideo", new loadDualVideo());

        //experimental - not used - shouldn't need to implement for Android
        functionMap.put("toggleDual", new toggleDual());


        /*** VIEWS AND TRANSITIONS ***/
        //These calls perform tasks using views and native functionality not provided by adobe air.

        //starts playing the game intro - it's a splash screen type thing with audio
        functionMap.put("startFirstScreen", new startFirstScreen());

        //stops the intro - called once the menu is loaded
        functionMap.put("stopFirstScreen", new stopFirstScreen());
        
        //disposes of the intro
        functionMap.put("endFirstScreen", new endFirstScreen());


        //shows a loading circle
        functionMap.put("shouldShowLoadingCircle", new shouldShowLoadingCircle());

        //fades the adobe air view to black or full opacity - used for scene transitions and loading
        functionMap.put("setGlobalAlpha", new setGlobalAlpha());


        //plays the outro / credits video - it's in mp4 format instead of .bik so it could be better compressed
        functionMap.put("playAV", new playAV());
        
        //stops the outro / credits video
        functionMap.put("stopAV", new stopAV());

        //since the outro video is being played outside of adobe air, this displays a home button over top of the outro video
        functionMap.put("setShowVideoHome", new setShowVideoHome());
        
        //once the outro video is done or skipped, returns the views to normal so that the main menu can be displayed
        functionMap.put("restart", new restart());

        //sets immersive mode
        functionMap.put("setImmersiveMode", new setImmersiveMode());

        //gets how much memory should be available to the game
        functionMap.put("getProcessMem", new getProcessMem());

        //requests write external storage permission
        functionMap.put("requestWriteExternal", new requestWriteExternal());

        /*** GOOGLE PLAY ***/
        //These calls have tasks related to Google Play.

        //submits an achievement
        functionMap.put("submitAchievement", new submitAchievement());

        //displays the achievement UI
        functionMap.put("showAchievements", new showAchievements());

        //decides if a rate game popup should be displayed
        //the BinkANE side of this seems to deal more with iCloud moreso, but the actionscript will need to be checked
        //to ensure that the popup goes to the correct link and such
        functionMap.put("shouldShowRate", new shouldShowRate());

        
        /*** MISC ***/
        //Random stuff that needs native functionality.

        //figures out the current device language and returns the language code
        functionMap.put("getPreferredLocalisations", new getPreferredLocalisations());

        //takes a screenshot
        functionMap.put("takeScreenshot", new takeScreenshot());

        //cancels the screenshot taking process
        functionMap.put("cancelScreenshot", new cancelScreenshot());

        //saves the screenshot to the user's camera roll
        functionMap.put("downloadScreenshot", new downloadScreenshot());

        //opens a native share dialog to share the screenshot
        functionMap.put("shareScreenshot", new shareScreenshot());

        //opens the NoodleNews more games popup
        functionMap.put("showMoreGames", new showMoreGames());
        

        /*** CLOUD SAVE ***/
        //These calls are all related to cloud save.
        //Adobe Air should already have some local save functionality.

        //fetches cloud save data
        functionMap.put("loadAllProfiles", new loadAllProfiles());

        //sets which cloud save is the current one
        functionMap.put("setProfilePos", new setProfilePos());

        //forces a given cloud save to be the current one
        functionMap.put("forceProfile", new forceProfile());

        //gets the chapter a player is on from the cloud save
        functionMap.put("getProfileChapter", new getProfileChapter());

        //gets what page of the handy manual the player is on from the cloud save
        functionMap.put("getProfileManualTip", new getProfileManualTip());

        //sets what page of the handy manual the player is on from the cloud save
        functionMap.put("setProfileManualTip", new setProfileManualTip());

        //sets which profile a user has selected
        functionMap.put("setSelectedProfile", new setSelectedProfile());

        //gets the currently selected profile
        functionMap.put("getSelectedProfile", new getSelectedProfile());

        //perform a cloud save
        functionMap.put("doSave", new doSave());

        //get specific level information from a cloud save, such as which objects have been interacted with in the scene
        functionMap.put("getResumeData", new getResumeData());


        /*** VIDEO RECORDING ***/
        //On iOS this was all done using ReplayKit.
        //Dan said it probably isn't worth doing on Android as it doesn't see much use.
        //Plus it would be a huge pain.

        //can the device record - returning false on Android
        functionMap.put("isRecordingAvailable", new isRecordingAvailable());

        //start a video recording - shouldn't need to implement for Android      
        functionMap.put("startRecord", new startRecord());

        //stop a video recording - shouldn't need to implement for Android
        functionMap.put("stopRecord", new stopRecord());

        //returns whether or not a video is currently being recorded - returning false on Android
        functionMap.put("isCurrentlyRecording", new isCurrentlyRecording());


        /*** APPLE TV ***/
        //The Apple TV version of the game required a lot of hacks to get it working in a very short timeframe.
        //Basically, for many of the menu systems, they have a hidden UI using Apple UIKit stuff, 
        //and they relay information about those back to the game. Additionally, the game did not contain all of the
        //assets in the initial download and would download chunks of the game on the fly using some crazy Apple system.

        //checks if the game is waiting for the chapter to be downloaded - returns false on Android
        functionMap.put("waitingForChapter", new waitingForChapter());

        //checks if the chapter is all downloaded - returns true on Android
        functionMap.put("isChapterLoaded", new isChapterLoaded());

        //checks if all chapters need to be available - returns false on Android
        functionMap.put("requireAllChapters", new requireAllChapters());

        //clears the focus of a UIKit menu - shouldn't need to implement for Android
        functionMap.put("focusClear", new focusClear());

        //adds focus to a UIKit menu element - shouldn't need to implement for Android
        functionMap.put("focusAdd", new focusAdd());

        //allows the game to think a finger is held down when the user clicks a button
        //and then removes the touch when the user clicks it again - shouldn't need to implement for Android
        functionMap.put("setMouseStartedDrag", new setMouseStartedDrag());

        //checks if a UIKit menu button has been clicked - shouldn't need to implement for Android
        functionMap.put("checkTVMenuClicked", new checkTVMenuClicked());

        //checks if a UIKit menu should be disposed of - shouldn't need to implement for Android
        functionMap.put("setTVMenuShouldExit", new setTVMenuShouldExit());

        //sets the TV menu mode - shouldn't need to implement for Android
        functionMap.put("setTVMenuMode", new setTVMenuMode());

        //gets actions performed on a UIKit menu - shouldn't need to implement for Android 
        functionMap.put("getTVMenuAction", new getTVMenuAction());


        /*** UNUSED ***/
        //These calls were only used for testing or special circumstances

        //clears the GLView - only used for testing - shouldn't need to implement for Android
        functionMap.put("clearTest", new clearTest());

        //checks if the app is in demo mode, which was used for an Apple in-store promotion
        //demo mode basically just resets the progress whenever the app is restarted - shouldn't need to implement for Android
        functionMap.put("isDemo", new isDemo());

		return functionMap;	
	}

    @Override  
    public void onActivityResult(int requestCode, int resultCode, Intent intent ) { 
        NoodleXBridge.onActivityResult(requestCode, resultCode, intent);
    }  
  
    @Override   
    public void onActivityStateChanged(ActivityState state) {  
        switch (state) {  
            case STARTED:  //onCreate
                if (!isCreated)
                {
                    NoodleXBridge.onCreate(getActivity().getIntent().getExtras(), getActivity());
                    NoodleXBridge.onStart();
                    isCreated = true;
                }
                break;
            case RESTARTED: //onRestart
                break;
            case RESUMED: //onResume
                NoodleXBridge.onResume();
                break;
            case PAUSED: //onPause
                NoodleXBridge.onPause();
                break;
            case STOPPED: //onStop
                NoodleXBridge.onStop();
                break;
            case DESTROYED: //onDestroy
                NoodleXBridge.onDestroy();
                break;
            default:
                break;
        }  
    }  
  
    @Override  
    public void onConfigurationChanged(Configuration paramConfiguration) {
    }  

    //usually you would just add this to a place or two in your activity, but Adobe AIR has to be diffucult about it...
    public void setImmersiveMode() {
        final Window window = getActivity().getWindow();
        final Window.Callback windowCallback = window.getCallback();
        final View decorView = window.getDecorView();
        final View.OnFocusChangeListener focusChangeListener = decorView.getOnFocusChangeListener();
        
        decorView.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                NoodleXBridge.onWindowFocusChanged(hasFocus);
            }
        });

        decorView.setOnSystemUiVisibilityChangeListener(new View.OnSystemUiVisibilityChangeListener() {
            @Override
            public void onSystemUiVisibilityChange(int visibility) {
                NoodleXBridge.onWindowFocusChanged(true);
            }
        });

        window.setCallback(new Window.Callback() {
            @Override
            public ActionMode onWindowStartingActionMode(Callback callback) {
                return windowCallback.onWindowStartingActionMode(callback);
            }
            
            @Override
            public void onWindowFocusChanged(boolean hasFocus) {
                NoodleXBridge.onWindowFocusChanged(hasFocus);
                
                try
                {
                    String type = hasFocus
                        ? "androidWindowFocusIn"
                        : "androidWindowFocusOut";
                    
                    dispatchStatusEventAsync(type, "");
                }
                catch (Exception e)
                {
                    // Ignore errors
                }
                
                windowCallback.onWindowFocusChanged(hasFocus);
            }
            
            @Override
            public void onWindowAttributesChanged(LayoutParams attrs) {
                windowCallback.onWindowAttributesChanged(attrs);
            }
            
            @Override
            public boolean onSearchRequested() {
                return windowCallback.onSearchRequested();
            }
            
            @Override
            public boolean onPreparePanel(int featureId, View view, Menu menu) {
                return windowCallback.onPreparePanel(featureId, view, menu);
            }
            
            @Override
            public void onPanelClosed(int featureId, Menu menu){
                windowCallback.onPanelClosed(featureId, menu);
            }
            
            @Override
            public boolean onMenuOpened(int featureId, Menu menu) {
                return windowCallback.onMenuOpened(featureId, menu);
            }
            
            @Override
            public boolean onMenuItemSelected(int featureId, MenuItem item) {
                return windowCallback.onMenuItemSelected(featureId, item);
            }
            
            @Override
            public void onDetachedFromWindow() {
                windowCallback.onDetachedFromWindow();
            }
            
            @Override
            public View onCreatePanelView(int featureId) {
                return windowCallback.onCreatePanelView(featureId);
            }
            
            @Override
            public boolean onCreatePanelMenu(int featureId, Menu menu) {
                return windowCallback.onCreatePanelMenu(featureId, menu);
            }
            
            @Override
            public void onContentChanged(){
                windowCallback.onContentChanged();
            }
            
            @Override
            public void onAttachedToWindow() {
                windowCallback.onAttachedToWindow();
            }
            
            @Override
            public void onActionModeStarted(ActionMode mode){
                windowCallback.onActionModeStarted(mode);
            }
            
            @Override
            public void onActionModeFinished(ActionMode mode) {
                windowCallback.onActionModeFinished(mode);
            }
            
            @Override
            public boolean dispatchTrackballEvent(MotionEvent event) {
                return windowCallback.dispatchTrackballEvent(event);
            }
            
            @Override
            public boolean dispatchTouchEvent(MotionEvent event) {
                return windowCallback.dispatchTouchEvent(event);
            }
            
            @Override
            public boolean dispatchPopulateAccessibilityEvent(AccessibilityEvent event) {
                return windowCallback.dispatchPopulateAccessibilityEvent(event);
            }
            
            @Override
            public boolean dispatchKeyShortcutEvent(KeyEvent event) {
                return windowCallback.dispatchKeyShortcutEvent(event);
            }
            
            @Override
            public boolean dispatchKeyEvent(KeyEvent event) {
                return windowCallback.dispatchKeyEvent(event);
            }
            
            @Override
            public boolean dispatchGenericMotionEvent(MotionEvent event) {
                return windowCallback.dispatchGenericMotionEvent(event);
            }

            @Override
            public ActionMode onWindowStartingActionMode(ActionMode.Callback callback, int type) {
                return windowCallback.onWindowStartingActionMode(callback, type);
            }

            @Override
            public boolean onSearchRequested(SearchEvent searchEvent) {
                return windowCallback.onSearchRequested(searchEvent);
            }
        });

        NoodleXBridge.onWindowFocusChanged(true);
    }
}
